#!/usr/bin/env python

"""
shellpreview.py

A thumbnail generator for KDE which runs an external tool to produce image
previews for files with appropriate MIME types.

Copyright (C) 2003-2005 David Boddie <david@boddie.org.uk>

This software is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this library; see the file COPYING
If not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

"""

# Only the QString and QImage classes are required from the Qt library
# and, even then, only implicitly.

#from qt import QString, QImage

# The ThumbCreator class is required since we will base our class on it.

import sip
from kio import ThumbCreator

import popen2


# Define a class which will be initialised when the plugin is loaded.
# The name must be defined in the accompanying details.py file.

class ShellPreviewClass(ThumbCreator):

    """ShellPreviewClass(ThumbCreator)
    
    In the default configuration, this previewer provide previews of
    files which Sketch (http://sketch.sourceforge.net/) can read by
    using the sk2ppm utility. The MIME types which this previewer
    accepts are specified in the template for the desktop file; this
    is defined in the details.py file.
    """

    def create(self, path, width, height, image):
    
        # Create a preview of the file at "path" in a new QImage object
        # and assign it to the "image" reference passed.
        # The size of the image is arbitrary, although "width" and
        # "height" can be used if desired.
        
        # Use the sk2ppm utility to create an image. This command will
        # output the image on stdout.
        
        stdout, stdin, stderr = popen2.popen3("sk2ppm '%s'" % path)
        
        # Read the output produced by the command.
        
        data = stdout.read()
        error = stderr.read()
        
        # Use a simple check for output and errors to determine whether
        # the utility was successful.
        
        if len(data) == 0 and error != "":
        
            # No data was produced and errors were reported.
            
            # Return 0 to indicate that a preview was not created.
            return 0
        
        # Load the data from the output into the image.
        
        image.loadFromData(data)
        
        # Return 1 on success; return 0 on failure.
        return 1
