/*
 * py_thumbcreator.cpp
 * 
 * Launch Python modules which return ThumbCreator instances for KDE's.
 * thumbnail generation mechanism.
 * 
 * Copyright (C) 2003-2005 Jim Bublitz, David Boddie <david@boddie.org.uk>
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this library; see the file COPYING
 * If not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <Python.h>
#include <kio/thumbcreator.h>
#include <klibloader.h>
#include "py_thumbcreator.h"


#define debug 0

ThumbCreator* report_error(char *msg)
{
    #if debug == 1
    FILE *f = fopen("/tmp/logs/thumb-debug.txt", "a");
    fprintf (f, "error: %s\n", msg);
    fclose(f);
    #endif
    return NULL;
}

PyObject* call_function(PyObject *callable, PyObject *args)
{
    PyObject *result = NULL, *pArgs;
    
    if (callable == NULL)
    {
        return NULL;
    }
    
    if (PyCallable_Check(callable))
    {
        if (args == NULL)
            pArgs = PyTuple_New(0);
        else
            pArgs = args;
        
        result = PyObject_CallObject(callable, pArgs);
        
        /* If the arguments were created in this function then decrease
           their reference count. */
        if (args == NULL)
            Py_XDECREF(pArgs);
            /* pDict and pFunc are borrowed and must not be Py_DECREF-ed */
        
        if (result == NULL)
        {
            #if debug == 1
            PyErr_Print();
            #else
            PyErr_Clear();
            #endif
        }
    }
    
    return result;
}

extern "C"
{
    ThumbCreator *new_creator( void )
    {
        // Declare a Python object to represent the object returned by the
        // Python module and its C++ equivalent.
        PyObject *pyThumbCreator;
        ThumbCreator *thumbCreator;
        
        KLibLoader::self()->globalLibrary(PYTHON_LIB_PATH);
        
        Py_Initialize();
        
        // Locate the path to the script and append it to sys.path list.
        
        PyRun_SimpleString("import sys\n");
        PyRun_SimpleString("sys.path.append('"MODULE_DIR"')\n");
    
        PyObject *pyModule = PyImport_ImportModule(MODULE_NAME);
        /* pyModule is a new reference */
        
        if (!pyModule) return report_error ("***failed to import module\n");
        
        PyObject *pyDict = PyModule_GetDict(pyModule);
        /* pDict is a borrowed reference */
        
        Py_XDECREF(pyModule);

        PyRun_String("import sip\n"
                     "def __embedded_factory__init__():\n"
                     "    global factory\n"
                     "    factory = "FACTORY"()\n"
                     "    return sip.unwrapinstance(factory)\n",
                     Py_file_input, pyDict, pyDict);

        // Get the Python module's factory function from the header file at
        // compile time.
        PyObject *pyFactory = PyDict_GetItemString(pyDict, "__embedded_factory__init__");
        /* pyFactory is a borrowed reference */

        // Set up the args for the Python factory function call.
        PyObject *args = NULL;
    
        // Instantiate the class.
        pyThumbCreator = call_function (pyFactory, args);
        /* pyThumbCreator should be a new reference to a long object. */

        if (pyThumbCreator)
        {
            // Convert the PyObject to a C++ object. *
            thumbCreator = (ThumbCreator*)PyLong_AsVoidPtr(pyThumbCreator);
        }
        
        Py_XDECREF(pyThumbCreator);
        
        return thumbCreator;
    }
};
